import { FrameButton, Frame, TransactionTargetResponseSendTransaction, TransactionTargetResponseSignTypedDataV4, SupportedParsingSpecification, FrameButtonLink, FrameButtonPost, FrameButtonTx, getFrame } from 'frames.js';
import { Dispatch } from 'react';
import { ParseResult } from 'frames.js/frame-parsers';
import { CastActionMessageResponse, ErrorMessageResponse, ComposerActionState, CastActionResponse, ComposerActionFormResponse } from 'frames.js/types';
import { CastId, FrameActionMessage } from '@farcaster/core';

type FarcasterSignerState<TSignerType = FarcasterSigner | null> = SignerStateInstance<TSignerType>;
type FarcasterSignerPendingApproval = {
    status: "pending_approval";
    deadline: number;
    privateKey: string;
    publicKey: string;
    requestFid: number;
    requestSigner: string;
    signature: string;
    signerApprovalUrl: string;
    token: string;
};
type FarcasterSignerImpersonating = {
    status: "impersonating";
    fid: number;
    privateKey: string;
    publicKey: string;
};
type FarcasterSignerApproved = {
    status: "approved";
    fid: number;
    privateKey: string;
    publicKey: string;
};
type FarcasterSigner = FarcasterSignerPendingApproval | FarcasterSignerImpersonating | FarcasterSignerApproved;
declare const mockFarcasterSigner: FarcasterSigner;

type FarcasterFrameContext = {
    /** Connected address of user, only sent with transaction data request */
    address?: `0x${string}`;
    castId: {
        hash: `0x${string}`;
        fid: number;
    };
};
/** Creates a frame action for use with `useFrame` and a proxy */
declare const signFrameAction: ({ buttonIndex, frameContext, frameButton, signer, target, inputText, state, transactionId, url, }: {
    target?: string;
    frameButton: FrameButton;
    buttonIndex: number;
    url: string;
    signer: FarcasterSignerState["signer"];
    inputText?: string;
    state?: string;
    transactionId?: `0x${string}`;
    frameContext: FarcasterFrameContext;
}) => Promise<{
    body: FrameActionBodyPayload;
    searchParams: URLSearchParams;
}>;
declare function createFrameActionMessageWithSignerKey(signerKey: string, { fid, url, buttonIndex, castId, inputText, state, address, transactionId, }: {
    fid: number;
    url: Uint8Array;
    buttonIndex: number;
    inputText: Uint8Array | undefined;
    castId: CastId;
    state: Uint8Array | undefined;
    address: Uint8Array | undefined;
    transactionId: Uint8Array | undefined;
}): Promise<{
    message: null;
    trustedBytes: null;
} | {
    message: FrameActionMessage | null;
    trustedBytes: string;
}>;

declare function isParseResult(result: unknown): result is ParseResult;
type UseFrameStackOptions = {
    initialFrame?: Frame | ParseResult;
    initialFrameUrl?: string | null;
};
type FrameStackAPI = {
    clear: () => void;
    createGetPendingItem: (arg: {
        request: FrameGETRequest;
    }) => FrameStackGetPending;
    createPostPendingItem: (arg: {
        action: SignedFrameAction;
        request: FramePOSTRequest;
    }) => FrameStackPostPending;
    /**
     * Creates a pending item without dispatching it
     */
    createCastOrComposerActionPendingItem: (arg: {
        action: SignedFrameAction;
        request: FramePOSTRequest;
    }) => FrameStackPostPending;
    markCastMessageAsDone: (arg: {
        pendingItem: FrameStackPostPending;
        endTime: Date;
        response: Response;
        responseData: CastActionMessageResponse;
    }) => void;
    markCastFrameAsDone: (arg: {
        pendingItem: FrameStackPostPending;
        endTime: Date;
    }) => void;
    markComposerFormActionAsDone: (arg: {
        pendingItem: FrameStackPostPending;
        endTime: Date;
    }) => void;
    markAsDone: (arg: {
        pendingItem: FrameStackGetPending | FrameStackPostPending;
        endTime: Date;
        response: Response;
        frameResult: GetFrameResult;
    }) => void;
    markAsDoneWithRedirect: (arg: {
        pendingItem: FrameStackPostPending;
        endTime: Date;
        location: string;
        response: Response;
        responseBody: unknown;
    }) => void;
    markAsDoneWithErrorMessage: (arg: {
        pendingItem: FrameStackPostPending;
        endTime: Date;
        response: Response;
        responseData: ErrorMessageResponse;
    }) => void;
    markAsFailed: (arg: {
        pendingItem: FrameStackGetPending | FrameStackPostPending;
        endTime: Date;
        requestError: Error;
        response: Response | null;
        responseBody: unknown;
        responseStatus: number;
    }) => void;
    markAsFailedWithRequestError: (arg: {
        endTime: Date;
        pendingItem: FrameStackPostPending;
        error: Error;
        response: Response;
        responseBody: unknown;
    }) => void;
};
declare function useFrameStack({ initialFrame, initialFrameUrl, }: UseFrameStackOptions): [
    FramesStack,
    React.Dispatch<FrameReducerActions>,
    FrameStackAPI
];

type OnTransactionArgs = {
    transactionData: TransactionTargetResponseSendTransaction;
    frameButton: FrameButton;
    frame: Frame;
};
type OnTransactionFunc = (arg: OnTransactionArgs) => Promise<`0x${string}` | null>;
type OnSignatureArgs = {
    signatureData: TransactionTargetResponseSignTypedDataV4;
    frameButton: FrameButton;
    frame: Frame;
};
type OnSignatureFunc = (args: OnSignatureArgs) => Promise<`0x${string}` | null>;
type OnComposerFormActionFuncArgs = {
    form: ComposerActionFormResponse;
    cast: ComposerActionState;
};
type OnComposeFormActionFuncReturnType = {
    /**
     * Updated composer action state
     */
    composerActionState: ComposerActionState;
} | undefined;
/**
 * If the function resolves to undefined it means that the dialog was probably closed resulting in no operation at all.
 */
type OnComposerFormActionFunc = (arg: OnComposerFormActionFuncArgs) => Promise<OnComposeFormActionFuncReturnType>;
type UseFetchFrameSignFrameActionFunction<TSignerStateActionContext extends SignerStateActionContext<any, any>, TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload> = (arg: {
    actionContext: TSignerStateActionContext;
    /**
     * @defaultValue false
     */
    forceRealSigner?: boolean;
}) => Promise<SignedFrameAction<TFrameActionBodyType>>;
type UseFetchFrameOptions<TSignerStorageType = object, TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload, TFrameContextType extends FrameContext = FarcasterFrameContext> = {
    stackAPI: FrameStackAPI;
    stackDispatch: React.Dispatch<FrameReducerActions>;
    specification: SupportedParsingSpecification;
    /**
     * URL or path to the frame proxy handling GET requests.
     */
    frameGetProxy: string;
    /**
     * URL or path to the frame proxy handling POST requests.
     */
    frameActionProxy: string;
    /**
     * Extra payload to be sent with the POST request.
     */
    extraButtonRequestPayload?: Record<string, unknown>;
    signFrameAction: UseFetchFrameSignFrameActionFunction<SignerStateActionContext<TSignerStorageType, TFrameContextType>, TFrameActionBodyType>;
    onTransaction: OnTransactionFunc;
    onSignature: OnSignatureFunc;
    onComposerFormAction: OnComposerFormActionFunc;
    /**
     * This function can be used to customize how error is reported to the user.
     *
     * Should be memoized
     */
    onError?: (error: Error) => void;
    /**
     * Custom fetch compatible function used to make requests.
     *
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API
     */
    fetchFn: typeof fetch;
    /**
     * This function is called when the frame returns a redirect in response to post_redirect button click.
     */
    onRedirect: (location: URL) => void;
};
type UseFrameOptions<TSignerStorageType = object, TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload, TFrameContextType extends FrameContext = FarcasterFrameContext> = {
    /** skip frame signing, for frames that don't verify signatures */
    dangerousSkipSigning?: boolean;
    /** the route used to POST frame actions. The post_url will be added as a the `url` query parameter */
    frameActionProxy: string;
    /** the route used to GET the initial frame via proxy */
    frameGetProxy: string;
    /** an signer state object used to determine what actions are possible */
    signerState: SignerStateInstance<TSignerStorageType, TFrameActionBodyType, TFrameContextType>;
    /** the url of the homeframe, if null / undefined won't load a frame */
    homeframeUrl: string | null | undefined;
    /** the initial frame. if not specified will fetch it from the homeframeUrl prop */
    frame?: Frame | ParseResult;
    /** connected wallet address of the user */
    connectedAddress: `0x${string}` | undefined;
    /** a function to handle mint buttons */
    onMint?: (t: OnMintArgs) => void;
    /** a function to handle transaction buttons that returned transaction data from the target, returns the transaction hash or null */
    onTransaction?: OnTransactionFunc;
    /** A function to handle transaction buttons that returned signature data from the target, returns signature hash or null */
    onSignature?: OnSignatureFunc;
    /** the context of this frame, used for generating Frame Action payloads */
    frameContext: TFrameContextType;
    /**
     * Extra data appended to the frame action payload
     */
    extraButtonRequestPayload?: Record<string, unknown>;
    /**
     * Which specification to use for parsing the frame action payload
     *
     * @defaultValue 'farcaster'
     */
    specification?: SupportedParsingSpecification;
    /**
     * This function can be used to customize how error is reported to the user.
     */
    onError?: (error: Error) => void;
    /**
     * This function can be used to customize how the link button click is handled.
     */
    onLinkButtonClick?: (button: FrameButtonLink) => void;
} & Partial<Pick<UseFetchFrameOptions, "fetchFn" | "onRedirect" | "onComposerFormAction">>;
type SignerStateActionContext<SignerStorageType = object, FrameContextType extends FrameContext = FarcasterFrameContext> = {
    target?: string;
    frameButton: FrameButton;
    buttonIndex: number;
    url: string;
    inputText?: string;
    signer: SignerStorageType | null;
    state?: string;
    transactionId?: `0x${string}`;
    address?: `0x${string}`;
    /** Transacting address is not included in non-transaction frame actions */
    frameContext: FrameContextType | Omit<FrameContextType, "address">;
};
type SignedFrameAction<TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload> = {
    body: TFrameActionBodyType;
    searchParams: URLSearchParams;
};
type SignFrameActionFunction<TSignerStateActionContext extends SignerStateActionContext<any, any> = SignerStateActionContext, TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload> = (actionContext: TSignerStateActionContext) => Promise<SignedFrameAction<TFrameActionBodyType>>;
interface SignerStateInstance<TSignerStorageType = object, TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload, TFrameContextType extends FrameContext = FarcasterFrameContext> {
    signer: TSignerStorageType | null;
    /**
     * True only if signer is approved or impersonating
     */
    hasSigner: boolean;
    signFrameAction: SignFrameActionFunction<SignerStateActionContext<TSignerStorageType, TFrameContextType>, TFrameActionBodyType>;
    /** is loading the signer */
    isLoadingSigner: boolean;
    /** A function called when a frame button is clicked without a signer */
    onSignerlessFramePress: () => void;
    logout: () => void;
}
type FrameGETRequest = {
    method: "GET";
    url: string;
};
type FramePOSTRequest<TSignerStateActionContext extends SignerStateActionContext<any, any> = SignerStateActionContext> = {
    method: "POST";
    frameButton: FrameButtonPost | FrameButtonTx;
    signerStateActionContext: TSignerStateActionContext;
    isDangerousSkipSigning: boolean;
    /**
     * The frame that was the source of the button press.
     */
    sourceFrame: Frame;
};
type FrameRequest<TSignerStateActionContext extends SignerStateActionContext<any, any> = SignerStateActionContext> = FrameGETRequest | FramePOSTRequest<TSignerStateActionContext>;
type FrameStackBase = {
    timestamp: Date;
    /** speed in seconds */
    speed: number;
    responseStatus: number;
    responseBody: unknown;
    requestDetails: {
        body?: object;
        searchParams?: URLSearchParams;
    };
    url: string;
};
type FrameStackPostPending = {
    method: "POST";
    timestamp: Date;
    status: "pending";
    request: FramePOSTRequest;
    requestDetails: {
        body?: object;
        searchParams?: URLSearchParams;
    };
    url: string;
};
type FrameStackGetPending = {
    method: "GET";
    timestamp: Date;
    status: "pending";
    request: FrameGETRequest;
    requestDetails: {
        body?: object;
        searchParams?: URLSearchParams;
    };
    url: string;
};
type FrameStackPending = FrameStackGetPending | FrameStackPostPending;
type GetFrameResult = ReturnType<typeof getFrame>;
type FrameStackDone = FrameStackBase & {
    request: FrameRequest;
    response: Response;
    frameResult: GetFrameResult;
    status: "done";
};
type FrameStackDoneRedirect = FrameStackBase & {
    request: FramePOSTRequest;
    response: Response;
    location: string;
    status: "doneRedirect";
};
type FrameStackRequestError = FrameStackBase & {
    request: FrameRequest;
    response: Response | null;
    status: "requestError";
    requestError: Error;
};
type FrameStackMessage = FrameStackBase & {
    request: FramePOSTRequest;
    response: Response;
    status: "message";
    message: string;
    type: "info" | "error";
};
type FramesStackItem = FrameStackPending | FrameStackDone | FrameStackDoneRedirect | FrameStackRequestError | FrameStackMessage;
type FramesStack = FramesStackItem[];
type FrameReducerActions = {
    action: "LOAD";
    item: FrameStackPending;
} | {
    action: "REQUEST_ERROR";
    pendingItem: FrameStackPending;
    item: FrameStackRequestError;
} | {
    action: "DONE_REDIRECT";
    pendingItem: FrameStackPending;
    item: FrameStackDoneRedirect;
} | {
    action: "DONE";
    pendingItem: FrameStackPending;
    item: FramesStackItem;
} | {
    action: "CLEAR";
} | {
    action: "RESET_INITIAL_FRAME";
    resultOrFrame: ParseResult | Frame;
    homeframeUrl: string | null | undefined;
};
type ButtonPressFunction<TSignerStateActionContext extends SignerStateActionContext<any, any>> = (frame: Frame, frameButton: FrameButton, index: number, fetchFrameOverride?: FetchFrameFunction<TSignerStateActionContext>) => void | Promise<void>;
type CastActionButtonPressFunctionArg = {
    castAction: CastActionResponse & {
        /** URL to cast action handler */
        url: string;
    };
    /**
     * @defaultValue false
     */
    clearStack?: boolean;
};
type CastActionButtonPressFunction = (arg: CastActionButtonPressFunctionArg) => Promise<void>;
type ComposerActionButtonPressFunctionArg = {
    castAction: CastActionResponse & {
        /** URL to cast action handler */
        url: string;
    };
    composerActionState: ComposerActionState;
    /**
     * @defaultValue false
     */
    clearStack?: boolean;
};
type ComposerActionButtonPressFunction = (arg: ComposerActionButtonPressFunctionArg) => Promise<void>;
type CastActionRequest<TSignerStateActionContext extends SignerStateActionContext<any, any> = SignerStateActionContext> = Omit<FramePOSTRequest, "method" | "frameButton" | "sourceFrame" | "signerStateActionContext"> & {
    method: "CAST_ACTION";
    action: CastActionResponse & {
        url: string;
    };
    signerStateActionContext: Omit<FramePOSTRequest<TSignerStateActionContext>["signerStateActionContext"], "frameButton" | "inputText" | "state">;
};
type ComposerActionRequest<TSignerStateActionContext extends SignerStateActionContext<any, any> = SignerStateActionContext> = Omit<FramePOSTRequest, "method" | "frameButton" | "sourceFrame" | "signerStateActionContext"> & {
    method: "COMPOSER_ACTION";
    action: CastActionResponse & {
        url: string;
    };
    composerActionState: ComposerActionState;
    signerStateActionContext: Omit<FramePOSTRequest<TSignerStateActionContext>["signerStateActionContext"], "frameButton" | "inputText" | "state">;
};
type FetchFrameFunction<TSignerStateActionContext extends SignerStateActionContext<any, any> = SignerStateActionContext> = (request: FrameRequest<TSignerStateActionContext> | CastActionRequest<TSignerStateActionContext> | ComposerActionRequest<TSignerStateActionContext>, 
/**
 * If true, the frame stack will be cleared before the new frame is loaded
 *
 * @defaultValue false
 */
shouldClear?: boolean) => Promise<void>;
type FrameState<TSignerStorageType = object, TFrameContextType extends FrameContext = FarcasterFrameContext> = {
    fetchFrame: FetchFrameFunction<SignerStateActionContext<TSignerStorageType, TFrameContextType>>;
    clearFrameStack: () => void;
    dispatchFrameStack: Dispatch<FrameReducerActions>;
    /** The frame at the top of the stack (at index 0) */
    currentFrameStackItem: FramesStackItem | undefined;
    /** A stack of frames with additional context, with the most recent frame at index 0 */
    framesStack: FramesStack;
    inputText: string;
    setInputText: (s: string) => void;
    onButtonPress: ButtonPressFunction<SignerStateActionContext<TSignerStorageType, TFrameContextType>>;
    homeframeUrl: string | null | undefined;
    onCastActionButtonPress: CastActionButtonPressFunction;
    onComposerActionButtonPress: ComposerActionButtonPressFunction;
};
type OnMintArgs = {
    target: string;
    frameButton: FrameButton;
    frame: Frame;
};
declare const themeParams: readonly ["bg", "buttonColor", "buttonBg", "buttonBorderColor", "buttonRadius", "buttonHoverBg"];
type FrameTheme = Partial<Record<(typeof themeParams)[number], string>>;
type FrameActionBodyPayload = Record<string, unknown>;
type FrameContext = Record<string, unknown>;

export { type FrameStackDoneRedirect as A, type FrameStackRequestError as B, type FrameStackMessage as C, type FramesStackItem as D, type FramesStack as E, type FarcasterFrameContext as F, type GetFrameResult as G, type FrameReducerActions as H, type CastActionButtonPressFunction as I, type ComposerActionButtonPressFunction as J, type CastActionRequest as K, type ComposerActionRequest as L, type FetchFrameFunction as M, type FrameState as N, type OnTransactionArgs as O, type OnMintArgs as P, themeParams as Q, type FrameTheme as R, type SignerStateActionContext as S, type FrameActionBodyPayload as T, type UseFetchFrameSignFrameActionFunction as U, type FrameContext as V, isParseResult as W, type FrameStackAPI as X, useFrameStack as Y, type FarcasterSignerState as a, type FarcasterSignerPendingApproval as b, createFrameActionMessageWithSignerKey as c, type FarcasterSignerImpersonating as d, type FarcasterSignerApproved as e, type FarcasterSigner as f, type OnTransactionFunc as g, type OnSignatureArgs as h, type OnSignatureFunc as i, type OnComposeFormActionFuncReturnType as j, type OnComposerFormActionFunc as k, type UseFetchFrameOptions as l, mockFarcasterSigner as m, type UseFrameOptions as n, type SignedFrameAction as o, type SignFrameActionFunction as p, type SignerStateInstance as q, type FrameGETRequest as r, signFrameAction as s, type FramePOSTRequest as t, type FrameRequest as u, type FrameStackBase as v, type FrameStackPostPending as w, type FrameStackGetPending as x, type FrameStackPending as y, type FrameStackDone as z };
